#include "daisy_terrarium.h"

#ifndef SAMPLE_RATE
//#define SAMPLE_RATE DSY_AUDIO_SAMPLE_RATE
#define SAMPLE_RATE 48014.f
#endif

using namespace daisy;

// # Rev3 and Rev4 with newest pinout.
// Compatible with Seed Rev3 and Rev4
constexpr Pin FSW_1_PIN = seed::D25;
constexpr Pin FSW_2_PIN = seed::D26;
constexpr Pin TOGGLE_1_PIN = seed::D10;
constexpr Pin TOGGLE_2_PIN = seed::D9;
constexpr Pin TOGGLE_3_PIN = seed::D8;
constexpr Pin TOGGLE_4_PIN = seed::D7;

constexpr Pin LED_1_R_PIN = seed::D22;
constexpr Pin LED_2_R_PIN = seed::D23;
//Terrarium doesn't have RGB LEDs, G and B are placeholders for LED functions
constexpr Pin LED_1_G_PIN = seed::D24;
constexpr Pin LED_1_B_PIN = seed::D24;
constexpr Pin LED_2_G_PIN = seed::D24;
constexpr Pin LED_2_B_PIN = seed::D24;

constexpr Pin KNOB_1_PIN = seed::D16;
constexpr Pin KNOB_2_PIN = seed::D17;
constexpr Pin KNOB_3_PIN = seed::D18;
constexpr Pin KNOB_4_PIN = seed::D19;
constexpr Pin KNOB_5_PIN = seed::D20;
constexpr Pin KNOB_6_PIN = seed::D21;

/*
// Leaving in place until older hardware is totally deprecated.
#ifndef SEED_REV2

// Rev2 Pinout
// Compatible with Seed rev1 and rev2

#define SW_1_PIN 28
#define SW_2_PIN 29

#define ENC_A_PIN 27
#define ENC_B_PIN 26
#define ENC_CLICK_PIN 14

#define LED_1_R_PIN 21
#define LED_1_G_PIN 20
#define LED_1_B_PIN 19
#define LED_2_R_PIN 0
#define LED_2_G_PIN 25
#define LED_2_B_PIN 24

#else

// Rev1 Pinout
// Compatible with Seed rev1 and rev2

#define SW_1_PIN 29
#define SW_2_PIN 28

#define ENC_A_PIN 27
#define ENC_B_PIN 26
#define ENC_CLICK_PIN 1

#define LED_1_R_PIN 21
#define LED_1_G_PIN 20
#define LED_1_B_PIN 19
#define LED_2_R_PIN 0
#define LED_2_G_PIN 25
#define LED_2_B_PIN 24

#endif
*/


using namespace daisy;

void Terrarium::Init(bool boost)
{
    // Set Some numbers up for accessors.
    // Initialize the hardware.
    seed.Configure();
    seed.Init(boost);
    InitSwitches();
    InitLeds();
    InitKnobs();
    SetAudioBlockSize(48);
}

void Terrarium::DelayMs(size_t del)
{
    seed.DelayMs(del);
}


void Terrarium::SetHidUpdateRates()
{
    for(int i = 0; i < KNOB_LAST; i++)
    {
        knobs[i]->SetSampleRate(AudioCallbackRate());
    }
}

void Terrarium::StartAudio(AudioHandle::InterleavingAudioCallback cb)
{
    seed.StartAudio(cb);
}

void Terrarium::StartAudio(AudioHandle::AudioCallback cb)
{
    seed.StartAudio(cb);
}

void Terrarium::ChangeAudioCallback(AudioHandle::InterleavingAudioCallback cb)
{
    seed.ChangeAudioCallback(cb);
}

void Terrarium::ChangeAudioCallback(AudioHandle::AudioCallback cb)
{
    seed.ChangeAudioCallback(cb);
}

void Terrarium::StopAudio()
{
    seed.StopAudio();
}

void Terrarium::SetAudioBlockSize(size_t size)
{
    seed.SetAudioBlockSize(size);
    SetHidUpdateRates();
}

size_t Terrarium::AudioBlockSize()
{
    return seed.AudioBlockSize();
}

void Terrarium::SetAudioSampleRate(SaiHandle::Config::SampleRate samplerate)
{
    seed.SetAudioSampleRate(samplerate);
    SetHidUpdateRates();
}

float Terrarium::AudioSampleRate()
{
    return seed.AudioSampleRate();
}

float Terrarium::AudioCallbackRate()
{
    return seed.AudioCallbackRate();
}

void Terrarium::StartAdc()
{
    seed.adc.Start();
}

void Terrarium::StopAdc()
{
    seed.adc.Stop();
}


void Terrarium::ProcessAnalogControls()
{
    knob1.Process();
    knob2.Process();
    knob3.Process();
    knob4.Process();
    knob5.Process();
    knob6.Process();
}

float Terrarium::GetKnobValue(Knob k)
{
    size_t idx;
    idx = k < KNOB_LAST ? k : KNOB_1;
    return knobs[idx]->Value();
}

void Terrarium::ClearLeds()
{
    // Using Color
    Color c;
    c.Init(Color::PresetColor::OFF);
    led1.SetColor(c);
    led2.SetColor(c);
    // Without
    // led1.Set(0.0f, 0.0f, 0.0f);
    // led2.Set(0.0f, 0.0f, 0.0f);
}

void Terrarium::UpdateLeds()
{
    led1.Update();
    led2.Update();
}

void Terrarium::InitSwitches()
{
    fsw1.Init(FSW_1_PIN);
    fsw2.Init(FSW_2_PIN);
    toggle1.Init(TOGGLE_1_PIN);
    toggle2.Init(TOGGLE_2_PIN);
    toggle3.Init(TOGGLE_3_PIN);
    toggle4.Init(TOGGLE_4_PIN);

    
    switches[FSW_1] = &fsw1;
    switches[FSW_2] = &fsw2;
    switches[TOGGLE_1] = &toggle1;
    switches[TOGGLE_2] = &toggle2;
    switches[TOGGLE_3] = &toggle3;
    switches[TOGGLE_4] = &toggle4;
    
}

void Terrarium::InitLeds()
{
    // LEDs are just going to be on/off for now.
    // TODO: Add PWM support
    led1.Init(LED_1_R_PIN, LED_1_G_PIN, LED_1_B_PIN, true);

    led2.Init(LED_2_R_PIN, LED_2_G_PIN, LED_2_B_PIN, true);

    ClearLeds();
    UpdateLeds();
}
void Terrarium::InitKnobs()
{
    // Configure the ADC channels using the desired pin
    AdcChannelConfig knob_init[KNOB_LAST];
    knob_init[KNOB_1].InitSingle(KNOB_1_PIN);
    knob_init[KNOB_2].InitSingle(KNOB_2_PIN);
    knob_init[KNOB_3].InitSingle(KNOB_3_PIN);
    knob_init[KNOB_4].InitSingle(KNOB_4_PIN);
    knob_init[KNOB_5].InitSingle(KNOB_5_PIN);
    knob_init[KNOB_6].InitSingle(KNOB_6_PIN);
    // Initialize with the knob init struct w/ 2 members
    // Set Oversampling to 32x
    seed.adc.Init(knob_init, KNOB_LAST);
    // Make an array of pointers to the knobs.
    knobs[KNOB_1] = &knob1;
    knobs[KNOB_2] = &knob2;
    knobs[KNOB_3] = &knob3;
    knobs[KNOB_4] = &knob4;
    knobs[KNOB_5] = &knob5;
    knobs[KNOB_6] = &knob6;
    for(int i = 0; i < KNOB_LAST; i++)
    {
        knobs[i]->Init(seed.adc.GetPtr(i), seed.AudioCallbackRate());
    }
}
